﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Animator))]
public class FrankController : MonoBehaviour
{
  Animator animator;

  public Limb ForeArm;
  public Limb BackArm;
  public Limb ForeLeg;
  public Limb BackLeg;

  Dictionary<LimbType, Limb> limbs;

  [HideInInspector]
  public int Health;

  void Start()
  {
    animator = GetComponent<Animator>();

    limbs = new Dictionary<LimbType, Limb>();
    limbs[LimbType.BACK_ARM] = BackArm;
    limbs[LimbType.FORE_ARM] = ForeArm;
    limbs[LimbType.BACK_LEG] = BackLeg;
    limbs[LimbType.FORE_LEG] = ForeLeg;
  }

  public bool HasStrikeLimb(MoveType move)
  {
    if (!Moves.IsStrike(move)) {
      return false;
    }

    return limbs[GetStrikeLimbType(move)] != null;
  }

  public bool HasDefenseLimb(MoveType move)
  {
    if (Moves.IsStrike(move)) {
      return false;
    }

    return limbs[GetDefenseLimbType(move)] != null;
  }

  public void Strike(MoveType selfMove, MoveType oponentMove, bool canOponentDefend)
  {
    LimbType type;
    if (Moves.IsStrike(selfMove)) {
      type = GetStrikeLimbType(selfMove);
    } else {
      type = GetDefenseLimbType(selfMove);
    }
    Limb limb = limbs[type];

    if (limb != null) {
      animator.SetTrigger(Moves.ComboMoveToAnimatorTrigger(selfMove));

      if (canOponentDefend && Moves.Negates(oponentMove, selfMove)) {
        limbs[type] = null;
        if (limb != null) {
          StartCoroutine(DropLimb(limb));
        }
      }
    }
  }

  public void ReactTo(MoveType selfMove, MoveType oponentMove)
  {
    if (Moves.IsStrike(oponentMove) && !(HasDefenseLimb(selfMove) && Moves.Negates(selfMove, oponentMove))) {
      animator.SetTrigger(Moves.DamageToAnimatorTrigger(oponentMove));
      Health -= 1;
    }
  }

  LimbType GetStrikeLimbType(MoveType move)
  {
    switch (move) {
      case MoveType.STRIKE_BACK_ARM:
        return LimbType.BACK_ARM;
      case MoveType.STRIKE_FORE_ARM:
        return LimbType.FORE_ARM;
      case MoveType.STRIKE_BACK_LEG:
        return LimbType.BACK_LEG;
      case MoveType.STRIKE_FORE_LEG:
        return LimbType.FORE_LEG;
      default:
        Debug.LogError(string.Format("GetStrikeLimbType got wrong move {0}", move));
        return LimbType.FORE_ARM; // Returning whatever
    }
  }

  LimbType GetDefenseLimbType(MoveType move)
  {
    if (move == MoveType.DEFEND_HEAD) {
      if (limbs[LimbType.BACK_ARM] != null) {
        return LimbType.BACK_ARM;
      }
      return LimbType.FORE_ARM;
    }
    if (move == MoveType.DEFEND_BALLS) {
      if (limbs[LimbType.BACK_LEG] != null) {
        return LimbType.BACK_LEG;
      }
      return LimbType.FORE_LEG;
    }

    Debug.LogError(string.Format("GetDefenseLimbType got wrong move {0}", move));
    return LimbType.FORE_ARM;
  }

  IEnumerator DropLimb(Limb limb)
  {
    yield return new WaitForSeconds(0.3f); // TODO: to param

    Limb copy = Instantiate(limb, limb.transform.parent);
    copy.gameObject.name = "Detached_Limb";
    limb.gameObject.SetActive(false);
    copy.DropSelf(transform.localScale.x > 0);
  }
}
